@@ -1,6 +1,10 @@
 package Graph;
 
 use strict;
+use warnings;
+
+# Need to get my head around all those redefines! -NEILB
+no warnings 'redefine';
 
 BEGIN {
     if (0) { # SET THIS TO ZERO FOR TESTING AND RELEASES!
@@ -14,12 +18,18 @@ use Graph::AdjacencyMap qw(:flags :fields);
 
 use vars qw($VERSION);
 
-$VERSION = '0.94';
+$VERSION = '0.9704';
 
 require 5.006; # Weak references are absolutely required.
 
 my $can_deep_copy_Storable =
-    eval 'require Storable; require B::Deparse; $Storable::VERSION >= 2.05 && $B::Deparse::VERSION >= 0.61' && !$@;
+  eval {
+    require Storable;
+    require B::Deparse;
+    Storable->VERSION(2.05);
+    B::Deparse->VERSION(0.61);
+    1;
+  };
 
 sub _can_deep_copy_Storable () {
     return $can_deep_copy_Storable;
@@ -53,11 +63,15 @@ sub _P () { 7 } # Predecessors.
 my $Inf;
 
 BEGIN {
-    local $SIG{FPE}; 
+  if ($] >= 5.022) {
+    $Inf = eval '+"Inf"';
+  } else {
+    local $SIG{FPE};
     eval { $Inf = exp(999) } ||
 	eval { $Inf = 9**9**9 } ||
 	    eval { $Inf = 1e+999 } ||
 		{ $Inf = 1e+99 };  # Close enough for most practical purposes.
+  }
 }
 
 sub Infinity () { $Inf }
@@ -108,12 +122,17 @@ sub eq {
     "$_[0]" eq "$_[1]"
 }
 
+sub boolify {
+  1;  # Important for empty graphs: they stringify to "", which is false.
+}
+
 sub ne {
     "$_[0]" ne "$_[1]"
 }
 
 use overload
     '""' => \&stringify,
+    'bool' => \&boolify,
     'eq' => \&eq,
     'ne' => \&ne;
 
@@ -369,6 +388,9 @@ sub _union_find_add_vertex {
 
 sub add_vertex {
     my $g = shift;
+    if (@_ != 1) {
+      $g->expect_hypervertexed;
+    }
     if ($g->is_multivertexed) {
 	return $g->add_vertex_by_id(@_, _GEN_ID);
     }
@@ -487,6 +509,9 @@ sub _union_find_add_edge {
 
 sub add_edge {
     my $g = shift;
+    if (@_ != 2) {
+      $g->expect_hyperedged;
+    }
     if ($g->is_multiedged) {
 	unless (@_ == 2 || $g->is_hyperedged) {
 	    require Carp;
@@ -494,12 +519,6 @@ sub add_edge {
 	}
 	return $g->add_edge_by_id(@_, _GEN_ID);
     }
-    unless (@_ == 2) {
-	unless ($g->is_hyperedged) {
-	    require Carp;
-	    Carp::croak("Graph::add_edge: graph is not hyperedged");
-	}
-    }
     my @e = $g->_add_edge( @_ );
     $g->[ _E ]->set_path( @e );
     $g->[ _G ]++;
@@ -991,9 +1010,15 @@ sub delete_vertex {
     $g->expect_non_unionfind;
     my $V = $g->[ _V ];
     return $g unless $V->has_path( @_ );
-    my $E = $g->[ _E ];
-    for my $e ( $g->_edges_at( @_ ) ) {
-	$E->_del_id( $e->[ 0 ] );
+    if (@_ == 1 && !($g->[ _f ] & (_HYPER|_REF|_UNIQ))) {
+      $g->delete_edge($_[0], $_) for $g->successors($_[0]);
+      $g->delete_edge($_, $_[0]) for $g->predecessors($_[0]);
+    } else {
+      # TODO: _edges_at is excruciatingly slow (rt.cpan.org 92427)
+      my $E = $g->[ _E ];
+      for my $e ( $g->_edges_at( @_ ) ) {
+        $E->_del_id( $e->[ 0 ] );
+      }
     }
     $V->del_path( @_ );
     $g->[ _G ]++;
@@ -1258,6 +1283,8 @@ sub has_cycle {
     @_ ? ($g->has_path(@_, $_[0]) ? 1 : 0) : 0;
 }
 
+*has_this_cycle = \&has_cycle;
+
 sub has_a_cycle {
     my $g = shift;
     my @r = ( back_edge => \&Graph::Traversal::has_a_cycle );
@@ -1816,6 +1843,28 @@ sub complement_graph {
 
 *complete = \&complete_graph;
 
+sub subgraph {
+  my ($g, $src, $dst) = @_;
+  $dst = $src unless defined $dst;
+  unless (ref $src eq 'ARRAY' && ref $dst eq 'ARRAY') {
+    Carp::croak("Graph::subgraph: need src and dst array references");
+  }
+  my $s = $g->new;
+  my @u = grep { $g->has_vertex($_) } @$src;
+  my @v = grep { $g->has_vertex($_) } @$dst;
+  $s->add_vertices(@u, @v);
+  for my $u (@u) {
+    my @e;
+    for my $v (@v) {
+      if ($g->has_edge($u, $v)) {
+        push @e, [$u, $v];
+      }
+    }
+    $s->add_edges(@e);
+  }
+  return $s;
+}
+
 ###
 # Transitivity.
 #
@@ -2077,6 +2126,13 @@ sub _expected {
     }
 }
 
+sub expect_no_args {
+    my $g = shift;
+    return unless @_;
+    my @caller1 = caller(1);
+    die "$caller1[3]: expected no arguments, got " . scalar @_ . ", at $caller1[1] line $caller1[2]\n";
+}
+
 sub expect_undirected {
     my $g = shift;
     _expected('undirected') unless $g->is_undirected;
@@ -2100,6 +2156,16 @@ sub expect_dag {
     _expected('directed acyclic', "@got") if @got;
 }
 
+sub expect_hypervertexed {
+    my $g = shift;
+    _expected('hypervertexed') unless $g->is_hypervertexed;
+}
+
+sub expect_hyperedged {
+    my $g = shift;
+    _expected('hyperedged') unless $g->is_hyperedged;
+}
+
 sub expect_multivertexed {
     my $g = shift;
     _expected('multivertexed') unless $g->is_multivertexed;
@@ -2349,10 +2415,11 @@ sub _root_opt {
     my $next =
 	exists $opt{ next_root } ?
 	    $opt{ next_root } :
-		$opt{ next_alphabetic } ?
-		    \&_next_alphabetic :
-			$opt{ next_numeric } ? \&_next_numeric :
-			    \&_next_random;
+              $opt{ next_alphabetic } ?
+                \&_next_alphabetic :
+                  $opt{ next_numeric } ?
+                    \&_next_numeric :
+                      \&_next_random;
     my $code = ref $next eq 'CODE';
     my $attr = exists $opt{ attribute } ? $opt{ attribute } : $defattr;
     return ( \%opt, \%unseen, \@unseen, $r, $next, $code, $attr );
@@ -2365,7 +2432,7 @@ sub _heap_walk {
     my $HF = Heap071::Fibonacci->new;
 
     while (defined $r) {
-	# print "r = $r\n";
+        # print "r = $r\n";
 	$add->($g, $h, $HF, $r, $attr, $unseenh, $etc);
 	delete $unseenh->{ $r };
 	while (defined $HF->top) {
@@ -2383,6 +2450,7 @@ sub _heap_walk {
 	}
 	return $h unless defined $next;
 	$r = $code ? $next->( $g, $unseenh ) : shift @$unseena;
+        last unless defined $r;
     }
 
     return $h;
@@ -2595,19 +2663,22 @@ sub topological_sort {
 
 *toposort = \&topological_sort;
 
+sub _undirected_copy_compute {
+  my $g = shift;
+  my $c = Graph::Undirected->new;
+  for my $v ($g->isolated_vertices) { # TODO: if iv ...
+    $c->add_vertex($v);
+  }
+  for my $e ($g->edges05) {
+    $c->add_edge(@$e);
+  }
+  return $c;
+}
+
 sub undirected_copy {
     my $g = shift;
-
     $g->expect_directed;
-
-    my $c = Graph::Undirected->new;
-    for my $v ($g->isolated_vertices) { # TODO: if iv ...
-	$c->add_vertex($v);
-    }
-    for my $e ($g->edges05) {
-	$c->add_edge(@$e);
-    }
-    return $c;
+    return _check_cache($g, 'undirected', \&_undirected_copy_compute);
 }
 
 *undirected_copy_graph = \&undirected_copy;
@@ -2640,6 +2711,7 @@ my %_cache_type =
      'biconnectivity'      => '_bcc',
      'SPT_Dijkstra'        => '_spt_di',
      'SPT_Bellman_Ford'    => '_spt_bf',
+     'undirected'          => '_undirected',
     );
 
 sub _check_cache {
@@ -2694,6 +2766,11 @@ sub SPT_Bellman_Ford_clear_cache {
     _clear_cache($g, 'SPT_Bellman_Ford');
 }
 
+sub undirected_copy_clear_cache {
+    my $g = shift;
+    _clear_cache($g, 'undirected_copy');
+}
+
 ###
 # Connected components.
 #
@@ -2796,7 +2873,7 @@ sub same_connected_components {
 	my ($CCE, $CCI) = $g->_connected_components();
 	my $u = shift;
 	my $c = $CCE->{ $u };
-	for my $v ( @_) {
+	for my $v ( @_ ) {
 	    return 0
 		unless defined $CCE->{ $v } &&
 		       $CCE->{ $v } eq $c;
@@ -3082,175 +3159,121 @@ sub strongly_connected_graph {
 # Biconnectivity.
 #
 
-sub _make_bcc {
-    my ($S, $v, $c) = @_;
-    my %b;
-    while (@$S) {
-	my $t = pop @$S;
-	$b{ $t } = $t;
-	last if $t eq $v;
+sub _biconnectivity_out {
+  my ($state, $u, $v) = @_;
+  if (exists $state->{stack}) {
+    my @BC;
+    while (@{$state->{stack}}) {
+      my $e = pop @{$state->{stack}};
+      push @BC, $e;
+      last if defined $u && $e->[0] eq $u && $e->[1] eq $v;
     }
-    return [ values %b, $c ];
+    if (@BC) {
+      push @{$state->{BC}}, \@BC;
+    }
+  }
+}
+
+sub _biconnectivity_dfs {
+  my ($g, $u, $state) = @_;
+  $state->{num}->{$u} = $state->{dfs}++;
+  $state->{low}->{$u} = $state->{num}->{$u};
+  for my $v ($g->successors($u)) {
+    unless (exists $state->{num}->{$v}) {
+      push @{$state->{stack}}, [$u, $v];
+      $state->{pred}->{$v} = $u;
+      $state->{succ}->{$u}->{$v}++;
+      _biconnectivity_dfs($g, $v, $state);
+      if ($state->{low}->{$v} < $state->{low}->{$u}) {
+	$state->{low}->{$u} = $state->{low}->{$v};
+      }
+      if ($state->{low}->{$v} >= $state->{num}->{$u}) {
+	_biconnectivity_out($state, $u, $v);
+      }
+    } elsif (defined $state->{pred}->{$u} &&
+	     $state->{pred}->{$u} ne $v &&
+	     $state->{num}->{$v} < $state->{num}->{$u}) {
+      push @{$state->{stack}}, [$u, $v];
+      if ($state->{num}->{$v} < $state->{low}->{$u}) {
+	$state->{low}->{$u} = $state->{num}->{$v};
+      }
+    }
+  }
 }
 
 sub _biconnectivity_compute {
-    my $g = shift;
-    my ($opt, $unseenh, $unseena, $r, $next, $code, $attr) =
-	$g->_root_opt(@_);
-    return () unless defined $r;
-    my %P;
-    my %I;
+    my ($g) = @_;
+    my %state;
+    @{$state{BC}} = ();
+    @{$state{BR}} = ();
+    %{$state{V2BC}} = ();
+    %{$state{BC2V}} = ();
+    @{$state{AP}} = ();
+    $state{dfs} = 0;
+    my @u = _shuffle $g->vertices;
+    for my $u (@u) {
+      unless (exists $state{num}->{$u}) {
+	_biconnectivity_dfs($g, $u, \%state);
+	_biconnectivity_out(\%state);
+	delete $state{stack};
+      }
+    }
+
+    # Mark the components each vertex belongs to.
+    my $bci = 0;
+    for my $bc (@{$state{BC}}) {
+      for my $e (@$bc) {
+	for my $v (@$e) {
+	  $state{V2BC}->{$v}->{$bci}++;
+	}
+      }
+      $bci++;
+    }
+
+    # Any isolated vertices get each their own component.
     for my $v ($g->vertices) {
-	$I{ $v } = 0;
+      unless (exists $state{V2BC}->{$v}) {
+	$state{V2BC}->{$v}->{$bci++}++;
+      }
     }
-    $I{ $r } = 1;
-    my %U;
-    my %S; # Self-loops.
-    for my $e ($g->edges) {
+
+    for my $v ($g->vertices) {
+      for my $bc (keys %{$state{V2BC}->{$v}}) {
+	$state{BC2V}->{$bc}->{$v}->{$bc}++;
+      }
+    }
+
+    # Articulation points / cut vertices are the vertices
+    # which belong to more than one component.
+    for my $v (keys %{$state{V2BC}}) {
+      if (keys %{$state{V2BC}->{$v}} > 1) {
+	push @{$state{AP}}, $v;
+      }
+    }
+
+    # Bridges / cut edges are the components of two vertices.
+    for my $v (keys %{$state{BC2V}}) {
+      my @v = keys %{$state{BC2V}->{$v}};
+      if (@v == 2) {
+	push @{$state{BR}}, \@v;
+      }
+    }
+
+    # Create the subgraph components.
+    my @sg;
+    for my $bc (@{$state{BC}}) {
+      my %v;
+      my $w = Graph::Undirected->new();
+      for my $e (@$bc) {
 	my ($u, $v) = @$e;
-	$U{ $u }{ $v } = 0;
-	$U{ $v }{ $u } = 0;
-	$S{ $u } = 1 if $u eq $v;
-    }
-    my $i = 1;
-    my $v = $r;
-    my %AP;
-    my %L = ( $r => 1 );
-    my @S = ( $r );
-    my %A;
-    my @V = $g->vertices;
-
-    # print "V : @V\n";
-    # print "r : $r\n";
-
-    my %T; @T{ @V } = @V;
-
-    for my $w (@V) {
-	my @s = $g->successors( $w );
-	if (@s) {
-	    @s = grep { $_ eq $w ? ( delete $T{ $w }, 0 ) : 1 } @s;
-	    @{ $A{ $w } }{ @s } = @s;
-	} elsif ($g->predecessors( $w ) == 0) {
-	    delete $T{ $w };
-	    if ($w eq $r) {
-		delete $I { $r };
-		$r = $v = each %T;
-		if (defined $r) {
-		    %L = ( $r => 1 );
-		    @S = ( $r );
-		    $I{ $r } = 1;
-		    # print "r : $r\n";
-		}
-	    }
-	}
+	$v{$u}++;
+	$v{$v}++;
+	$w->add_edge($u, $v);
+      }
+      push @sg, [ keys %v ];
     }
 
-    # use Data::Dumper;
-    # print "T : ", Dumper(\%T);
-    # print "A : ", Dumper(\%A);
-
-    my %V2BC;
-    my @BR;
-    my @BC;
-
-    my @C;
-    my $Avok;
-
-    while (keys %T) {
-	# print "T = ", Dumper(\%T);
-	do {
-	    my $w;
-	    do {
-		my @w = _shuffle values %{ $A{ $v } };
-		# print "w = @w\n";
-		$w = first { !$U{ $v }{ $_ } } @w;
-		if (defined $w) {
-		    # print "w = $w\n";
-		    $U{ $v }{ $w }++;
-		    $U{ $w }{ $v }++;
-		    if ($I{ $w } == 0) {
-			$P{ $w } = $v;
-			$i++;
-			$I{ $w } = $i;
-			$L{ $w } = $i;
-			push @S, $w;
-			$v = $w;
-		    } else {
-			$L{ $v } = $I{ $w } if $I{ $w } < $L{ $v };
-		    }
-		}
-	    } while (defined $w);
-	    # print "U = ", Dumper(\%U);
-	    # print "P = ", Dumper(\%P);
-	    # print "L = ", Dumper(\%L);
-	    if (!defined $P{ $v }) {
-		# Do nothing.
-	    } elsif ($P{ $v } ne $r) {
-		if ($L{ $v } < $I{ $P{ $v } }) {
-		    $L{ $P{ $v } } = $L{ $v } if $L{ $v } < $L{ $P{ $v } };
-		} else {
-		    $AP{ $P{ $v } } = $P{ $v };
-		    push @C, _make_bcc(\@S, $v, $P{ $v } );
-		}
-	    } else {
-		my $e;
-		for my $w (_shuffle keys %{ $A{ $r } }) {
-		    # print "w = $w\n";
-		    unless ($U{ $r }{ $w }) {
-			$e = $r;
-			# print "e = $e\n";
-			last;
-		    }
-		}
-		$AP{ $e } = $e if defined $e;
-		push @C, _make_bcc(\@S, $v, $r);
-	    }
-	    # print "AP = ", Dumper(\%AP);
-	    # print "C  = ", Dumper(\@C);
-	    # print "L  = ", Dumper(\%L);
-	    $v = defined $P{ $v } ? $P{ $v } : $r;
-	    # print "v = $v\n";
-	    $Avok = 0;
-	    if (defined $v) {
-		if (keys %{ $A{ $v } }) {
-		    if (!exists $P{ $v }) {
-			for my $w (keys %{ $A{ $v } }) {
-			    $Avok++ if $U{ $v }{ $w };
-			}
-			# print "Avok/1 = $Avok\n";
-			$Avok = 0 unless $Avok == keys %{ $A{ $v } };
-			# print "Avok/2 = $Avok\n";
-		    }
-		} else {
-		    $Avok = 1;
-		    # print "Avok/3 = $Avok\n";
-		}
-	    }
-	} until ($Avok);
-
-	last if @C == 0 && !exists $S{$v};
-
-	for (my $i = 0; $i < @C; $i++) {
-	    for my $v (@{ $C[ $i ]}) {
-		$V2BC{ $v }{ $i }++;
-		delete $T{ $v };
-	    }
-	}
-
-	for (my $i = 0; $i < @C; $i++) {
-	    if (@{ $C[ $i ] } == 2) {
-		push @BR, $C[ $i ];
-	    } else {
-		push @BC, $C[ $i ];
-	    }
-	}
-
-	if (keys %T) {
-	    $r = $v = each %T;
-	}
-    }
-    
-    return [ [values %AP], \@BC, \@BR, \%V2BC ];
+    return [ $state{AP}, \@sg, $state{BR}, $state{V2BC}, ];
 }
 
 sub biconnectivity {
@@ -3263,26 +3286,26 @@ sub biconnectivity {
 
 sub is_biconnected {
     my $g = shift;
-    my ($ap, $bc) = ($g->biconnectivity(@_))[0, 1];
-    return defined $ap ? @$ap == 0 && $g->vertices >= 3 : undef;
+    my ($ap) = ($g->biconnectivity(@_))[0];
+    return $g->edges >= 2 ? @$ap == 0 : undef ;
 }
 
 sub is_edge_connected {
     my $g = shift;
     my ($br) = ($g->biconnectivity(@_))[2];
-    return defined $br ? @$br == 0 && $g->edges : undef;
+    return $g->edges >= 2 ? @$br == 0 : undef;
 }
 
 sub is_edge_separable {
     my $g = shift;
-    my $c = $g->is_edge_connected;
-    defined $c ? !$c && $g->edges : undef;
+    my ($br) = ($g->biconnectivity(@_))[2];
+    return $g->edges >= 2 ? @$br > 0 : undef;
 }
 
 sub articulation_points {
     my $g = shift;
     my ($ap) = ($g->biconnectivity(@_))[0];
-    return defined $ap ? @$ap : ();
+    return @$ap;
 }
 
 *cut_vertices = \&articulation_points;
@@ -3290,14 +3313,14 @@ sub articulation_points {
 sub biconnected_components {
     my $g = shift;
     my ($bc) = ($g->biconnectivity(@_))[1];
-    return defined $bc ? @$bc : ();
+    return @$bc;
 }
 
 sub biconnected_component_by_index {
     my $g = shift;
     my $i = shift;
     my ($bc) = ($g->biconnectivity(@_))[1];
-    return defined $bc ? $bc->[ $i ] : undef;
+    return $bc->[ $i ];
 }
 
 sub biconnected_component_by_vertex {
@@ -3407,7 +3430,9 @@ sub SPT_Dijkstra {
 	$opt{ first_root } = $first_root = $g->random_vertex();
     }
     my $spt_di = $g->get_graph_attribute('_spt_di');
-    unless (defined $spt_di && exists $spt_di->{ $first_root } && $spt_di->{ $first_root }->[ 0 ] == $g->[ _G ]) {
+    unless (defined $spt_di &&
+            exists $spt_di->{ $first_root } &&
+            $spt_di->{ $first_root }->[ 0 ] == $g->[ _G ]) {
 	my %etc;
 	my $sptg = $g->_heap_walk($g->new, \&_SPT_add, \%etc, %opt);
 	$spt_di->{ $first_root } = [ $g->[ _G ], $sptg ];
@@ -3729,7 +3754,7 @@ sub vertex_eccentricity {
 		$max = $l;
 	    }
 	}
-	return $max;
+	return defined $max ? $max : Infinity();
     } else {
 	return Infinity();
     }
@@ -3754,6 +3779,7 @@ sub shortest_path {
 		    @min = $t->path_vertices($u, $v);
 		}
 	    }
+            print "min/1 = @min\n";
 	    return wantarray ? @min : $min;
 	}
     } else {
@@ -3768,10 +3794,13 @@ sub shortest_path {
 		    @min = $t->path_vertices($u, $v);
 		}
 	    }
+            print "min/2 = @min\n";
 	    return wantarray ? @min : $min;
 	} else {
 	    my ($min, $max, $minp, $maxp) = $g->_minmax_path(@_);
-	    return defined $minp ? (wantarray ? @$minp : $min) : undef;
+	    return defined $minp
+              ? (wantarray ? @$minp : $min)
+              : wantarray ? () : undef;
 	}
     }
 }
@@ -3793,11 +3822,12 @@ sub center_vertices {
     $delta = 0 unless defined $delta;
     $delta = abs($delta);
     my @c;
+    my $Inf = Infinity();
     my $r = $g->radius;
-    if (defined $r) {
+    if (defined $r && $r != $Inf) {
 	for my $v ($g->vertices) {
 	    my $e = $g->vertex_eccentricity($v);
-	    next unless defined $e;
+	    next unless defined $e && $e != $Inf;
 	    push @c, $v if abs($e - $r) <= $delta;
 	}
     }
